searchData={"items":[{"type":"module","title":"Flake","doc":"Public-facing API for starting and generating flake ID's.","ref":"Flake.html"},{"type":"function","title":"Flake.get_flake_components/1","doc":"Given a flake ID, return a map that breaks down the different components of the flake ID.\n\nThis is particularly useful for troubleshooting or for use in automated tests.","ref":"Flake.html#get_flake_components/1"},{"type":"function","title":"Examples - Flake.get_flake_components/1","doc":"```elixir\niex> Flake.get_flake_components(1846405654575644672)\n%{time: 1719599268, machine_id: 1, worker_id: 1, counter: 0}\n```","ref":"Flake.html#get_flake_components/1-examples"},{"type":"function","title":"Flake.get_id/1","doc":"Generates a 64-bit unsigned integer flake ID given a worker ID.\n\nWorker ID must be within the range of 0..(N-1), where N is the total number of workers\npassed in to the `start/1` or `start/2` functions.\n\nFor best performance, try to evenly distribute calls to flake worker processes from\ndifferent calling processes.","ref":"Flake.html#get_id/1"},{"type":"function","title":"Flake.start/1","doc":"Creates worker processes and assigns the application's machine ID to all workers.\n\nThe number of worker processes defaults to the number of BEAM schedulers, or 64,\nwhichever is less.\n\nThis *must* be called before generating a flake ID.","ref":"Flake.html#start/1"},{"type":"function","title":"Flake.start/2","doc":"Creates worker processes and assigns the application's machine ID to all workers.\n\nThis *must* be called before generating a flake ID.","ref":"Flake.html#start/2"},{"type":"type","title":"Flake.flake_id/0","doc":"","ref":"Flake.html#t:flake_id/0"},{"type":"type","title":"Flake.machine_id/0","doc":"","ref":"Flake.html#t:machine_id/0"},{"type":"type","title":"Flake.worker_id/0","doc":"","ref":"Flake.html#t:worker_id/0"},{"type":"module","title":"Flake.Manager","doc":"","ref":"Flake.Manager.html"},{"type":"function","title":"Flake.Manager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Flake.Manager.html#child_spec/1"},{"type":"function","title":"Flake.Manager.reset/0","doc":"__Only use this in tests!__ Resets the entire state of the flake application.\n\nKills and restarts all worker processes. Resets its own state.\n\nReturns:\n    :ok","ref":"Flake.Manager.html#reset/0"},{"type":"extras","title":"Flake","doc":"# Flake\n\nGenerates a [Snowflake](https://en.wikipedia.org/wiki/Snowflake_ID) ID. This implementation is slightly different than the original\nsnowflake ID but still maintains its most important characteristics: 64-bit integer, unique ID generation within a\ndistributed cluster, and timestamp sortable.","ref":"readme.html"},{"type":"extras","title":"Usage - Flake","doc":"To generate a flake ID, you must explicitly \"start\" the system. This is separate from the typical BEAM\napplication startup. The flake application needs to know what the assigned machine ID is, as well as\nhow many worker processes to create. *You can't create more than 64 workers.*\n```elixir\nFlake.start(0, 20) # assigns a machine id of 0 and creates 20 worker processes\nFlake.start(0)     # assigns a machine id of 0 and defaults to the number of System.schedulers()\n```\n\nAfter starting up, you can generate a flake ID with:\n```elixir\n{:ok, flake} = Flake.get_id(20)\n```\nAs a performance optimization, you must explicitly request which worker to generate an id. If you started with\n20 worker processes, then the valid range is 1 - 20, inclusive. For troubleshooting or general testing,\nyou can get a breakdown of the flake components as a map:\n```elixir\n# returns %{time: timestamp, machine_id: machine_id, worker_id: worker_id, counter: counter}\ncomponents =\n    Flake.get_id(20)\n    |> Flake.get_flake_components()\n```","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Flake","doc":"The package can be installed\nby adding `flake` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:flake, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Implementation - Flake","doc":"Flake ID's are 64-bit, unsigned integers broken down into four separate components:\n\n| Name       | Bits |\n| ---------- | ---- |\n| timestamp  | 34   |\n| machine id | 8    |\n| worker id  | 6    |\n| counter    | 16   |","ref":"readme.html#implementation"},{"type":"extras","title":"Performance - Flake","doc":"Performance testing was performed on an Apple M3 Pro, 18GB memory, 11 cores.\n\nUsed the following script for testing:\n```elixir\nworkers = 20\nwork = div(1_000_000, workers)\nFlake.start(1, workers)\n\n{time, ids} =\n  :timer.tc(fn ->\n    ts =\n      for j <- 0..(workers - 1),\n          do:\n            Task.async(fn ->\n              for _i <- 1..work do\n                {:ok, id} = Flake.get_id(j)\n                id\n              end\n            end)\n\n    for t <- ts, do: Task.await(t)\n  end)\n```\nThe test consistently yields a result of 0.59 seconds per 1,000,000 ID's, about 1.6 - 1.7\nmillion per second. Theoretically, since no consensus is involved in generating an ID, adding servers would multiply\nthis result. More results will be posted in the future.","ref":"readme.html#performance"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}